/*
 * This file is part of "Obect Teams Development Tooling"-Software.
 *
 * Responsible Authors: OTDT Development Team, see file AUTHORS
 * Please visit http://www.objectteams.org for further information.
 *
 * Created on Dec 12, 2004
 *
 */
package org.objectteams;

/**
 * Signal a violation of OTJLD 2.4.1(c).
 * Also Team.getRole(Object) may throw a DuplicateRoleException if
 * more than one role is found for the given base object
 * (in that case those roles are found in different role-caches).
 * 
 * 
 * @author stephan
 * @version $Id: DuplicateRoleException.java,v 1.3 2006-10-16 13:34:04 resix Exp $
 */
public class DuplicateRoleException extends RuntimeException {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * @param message
	 */
	public DuplicateRoleException(String roleClassName) {
		super("Failed to create a role instance of type "+roleClassName+"\n"+
			  "A role for the given base object already exists (OTJLD 2.4.1(c)).");
	}
	
	public DuplicateRoleException(String roleName1, String roleName2) {
		super("Ambiguous role instances: found a role in hierarchies "+
				roleName1+" and "+roleName2);
	}
}
