package org.objectteams.transformer;

import de.fub.bytecode.generic.*;

// ObjectTeams/Java Runtime System.
//Copyright (c) 2003 Stephan Herrmann.
//Project Home Page: http://www.ObjectTeams.org
//
//This program is free software.
//
//You may redistribute it and/or modify it under the terms of the
//GNU Lesser General Public License as published by the
//Free Software Foundation. Version 2.1 of the license should be
//included with this distribution in the file LICENSE.LGPL.
//If the license is not included with this distribution,
//you may find a copy at the FSF web site at 'www.gnu.org' or
//'www.fsf.org', or you may write to the Free Software Foundation,
//675 Mass Ave, Cambridge, MA 02139 USA.
//
//THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
//NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
//OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
//CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
//REDISTRIBUTION OF THIS SOFTWARE.

public interface OTConstants {
	// ------------------------------------------
	// ---------- Types: ------------------------
	// ------------------------------------------
	/**                            Type <tt>java.lang.Object</tt> */
    ObjectType object        = new ObjectType("java.lang.Object");
	/**                            Type <tt>java.lang.Object</tt> */
    ObjectType string        = new ObjectType("java.lang.String");
    /**                            Type <tt>java.lang.Class</tt> */
    ObjectType classType = new ObjectType("java.lang.Class");
    /** 						   Signature of java.lang.Class#getMethod(String, Class...) */
    public static final Type[] getMethodSignature = new Type[]{string, new ArrayType(classType, 1)};
    /**                            Type <tt>java.lang.reflect.Method</tt> */
    ObjectType methodType = new ObjectType("java.lang.reflect.Method");
    /**                            Type <tt>org.objectteams.Team</tt> */
    ObjectType teamType          = new ObjectType("org.objectteams.Team");
	/**                            Type <tt>org.objectteams.LiftingVetoException</tt> */
    ObjectType liftingVeto   = new ObjectType("org.objectteams.LiftingVetoException");
    /**                            Type <tt>org.objectteams.transformer.OTREInternalError</tt> */	
    ObjectType internalError = new ObjectType("org.objectteams.transformer.OTREInternalError");
	/**                            Type <tt>org.objectteams.transformer.OTREInternalError</tt> */	
	ObjectType notProvidedError = new ObjectType("org.objectteams.ResultNotProvidedError");
	/**                            Type <tt>org.objectteams.transformer.UnsupportedFeatureException</tt> */	
	ObjectType unsupportedFeature = new ObjectType("org.objectteams.UnsupportedFeatureException");
	
	ObjectType threadType  = new ObjectType("java.lang.Thread");
	
	/**                          Type <tt>org.objectteams.Team[]</tt> */
    ArrayType  teamArray   = new ArrayType(teamType, 1);
	/**                          Type <tt>int[]</tt> */
    ArrayType  intArray    = new ArrayType(Type.INT, 1);
	/**                          Type <tt>java.lang.Object[]</tt> */
    ArrayType  objectArray = new ArrayType(object, 1);

	ObjectType roleSetType = new ObjectType("java.util.HashSet");
	
	ObjectType nullPointerException = new ObjectType("java.lang.NullPointerException");

	String STRING_BUFFER_NAME    = "java.lang.StringBuffer";
	
	// ============ VERSION: ==============
	public static final int    OT_VERSION_MAJOR = 0;
    public static final int    OT_VERSION_MINOR = 8;
    public static final int    OT_REVISION = 18;
    
    // required compiler revision in the 0.9 stream:
    public static final int    OT09_REVISION = 26;

    // required compiler revision in the 1.0 stream:
    public static final int    OT10_REVISION = 0;
    
    // required compiler revision in the 1.1 stream:
    public static final int    OT11_REVISION = 0;

    // required compiler revision in the 1.2 stream:
    public static final int    OT12_REVISION = 0;

    // required compiler revision in the 1.3 stream:
    public static final int    OT13_REVISION = 0;

    // ------------------------------------------
	// ---------- Flags and Modifiers: ----------
	// ------------------------------------------
	/** Bytecode encoding of modifier <tt>team</tt> */
    final static int TEAM  = 0x8000;

	// 'CallinFlags':
	final static int OVERRIDING =1; // this role method is inherited from the super role
	// FIXME(SH): no longer generated by the compiler:
	final static int WRAPPER =2; // this is a role method wrapper (in a team)

	// ------------------------------------------
	// ---------- Names: ------------------------
	// ------------------------------------------
	/**                              General prefix to mark all generated names. */
    final static String OT_PREFIX = "_OT$";
	/**                              Name of the base reference of roles. */
    final static String BASE      = "_OT$base";
	/**                              Name of the getBase method of roles (ifc and class). */
    final static String GET_BASE  = "_OT$getBase";
	/**							  Prefix for otdt. */
	final static String OTDT_PREFIX = "__OT__";
	/**							  Tsuper marker interface prefix. */
	final static String TSUPER_PREFIX = "TSuper__OT__";
	// -----------------------------------------
	// ---------- Signature enhancement --------
	// -----------------------------------------
	
	/**                              Name of synthetic parameter. */
	final static String TEAMS     = "_OT$teams";
	/**                              Name of synthetic parameter. */
	final static String TEAMIDS   = "_OT$teamIDs";
	/**                              Name of synthetic parameter. */
	final static String IDX       = "_OT$idx";
	/**                              Name of synthetic parameter. */
	final static String BIND_IDX       = "_OT$bindIdx";
	/**                              Name of synthetic parameter. */
	final static String UNUSED    = "_OT$unusedArgs";
	/**                              Name of synthetic parameter. */
    final static String BASE_METH_TAG = "_OT$baseMethTag";

	/**                            Number of extra arguments in enhanced signatures. */
	static final int EXTRA_ARGS  = 6;
	/**                            Position of generated argument. */
	static final int TEAMS_ARG   = 1;
	/**                            Position of generated argument. */
	static final int TEAMIDS_ARG = 2;
	/**                            Position of generated argument. */
	static final int IDX_ARG     = 3;
	/**                            Position of generated argument. */
	static final int BIND_IDX_ARG = 4;
	/**                            Position of generated argument. */
    static final int BASE_METH_ARG = 5; // ## really const? also UNUSED?
	/**                            Position of generated argument. */
	static final int UNUSED_ARG  = 6;

	// ---------- Features to prevent/aid garbage collection: ----------
	String ROLE_SET              = OT_PREFIX + "roleSet";    // field  HashSet _OT$roleSet;
	String ADD_ROLE              = OT_PREFIX + "addRole";    // method void _OT$addRole(Object)
	String REMOVE_ROLE           = OT_PREFIX + "removeRole"; // method void _OT$removeRole(Object)

	String IBOUND_BASE           = "org.objectteams.IBoundBase"; // interface comprising the above methods.
	
	
	// -----------------------------------------
	// ---------- Other constants --------
	// -----------------------------------------
	/**							Marker for comment lines in the team config file. */    
	static final String COMMENT_MARKER = "#";
	/**							Constant for invalid base method tags (a method can not be relocated from a base call). */    
	static final int INVALID_BASE_METHOD_TAG = -2;
	
	// --------------------------------------------------------------
	// ---------- Separator for static replace binding keys ---------
	// --------------------------------------------------------------
	 static final String STATIC_REPLACE_BINDING_SEPARATOR = ".."; 
	 
	 // ------------------------------------------------------------------------------------
	 // ---------- Linenumbers with more information.  ----------------------
	 // ---------- (semantic linenumber) For debugging purpose.  ------
	 // ------------------------------------------------------------------------------------
	 static final int STEP_OVER_LINENUMBER = Short.MAX_VALUE *2;
	 static final int STEP_INTO_LINENUMBER = STEP_OVER_LINENUMBER - 1;
}
