package org.objectteams.transformer.jmangler;

import java.util.Iterator;

import org.cs3.jmangler.supplier.UnextendableClassSet;
import org.cs3.jmangler.tau.ExtensionSet;
import org.objectteams.transformer.BaseCallRedirection;
import org.objectteams.transformer.ObjectTeamsTransformation.SharedState;

import de.fub.bytecode.generic.ClassGen;

//ObjectTeams/Java Runtime System.
//Copyright (c) 2002 Christine Hundt, 2003 Stephan Herrmann, 2004 Christine Hundt.
//Project Home Page: http://www.ObjectTeams.org
//
//This program is free software.
//
//You may redistribute it and/or modify it under the terms of the
//GNU Lesser General Public License as published by the
//Free Software Foundation. Version 2.1 of the license should be
//included with this distribution in the file LICENSE.LGPL.
//If the license is not included with this distribution,
//you may find a copy at the FSF web site at 'www.gnu.org' or
//'www.fsf.org', or you may write to the Free Software Foundation,
//675 Mass Ave, Cambridge, MA 02139 USA.
//
//THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
//NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
//OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
//CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
//REDISTRIBUTION OF THIS SOFTWARE.

/**
 * for callin-role-methods with recursive method-calls (base calls) we add a
 * method with extendend signature.
 * Within the extendend method recursive calls are replaced by the corresponding
 * (chaining)base-call.<p>
 * For example:
 * <pre>
 *      callin m1() { m1(); } -->
 *      callin m1(Team _OT$teams[], int _OT$teamIDs[],
 *                int _OT$idx, int _OT$baseMethTag){
 *          liftToRole(b1._OT$m1$chain(Team _OT$teams[],
 *                                     int _OT$teamIDs[],
 *                                     int _OT$idx,
 *                                     int _OT$baseMethTag));
 *      }
 * </pre>
 *
 * @version $Id: BaseCallRedirector.java,v 1.4 2006-10-09 16:00:05 resix Exp $
 * @author  Christine Hundt, Stephan Herrmann
 */
public class BaseCallRedirector extends ObjectTeamsTransformer {

    private BaseCallRedirection transformation = new BaseCallRedirection(new SharedState());
    
    public ExtensionSet transformInterface(UnextendableClassSet cs) {

        Iterator iter = cs.getIteratorForTransformableClasses();
        ExtensionSet es = new ExtensionSet();
        JManglerEnhancer jme = new JManglerEnhancer(this, es, cs);
        
        while (iter.hasNext()) {
            // look at every transformable class
            ClassGen cg = (ClassGen)iter.next();

            transformation.doTransformInterface(jme, cg);
        }
        // return the changes
        return es;
    }
}
