package org.objectteams.transformer.jmangler;

import java.util.Iterator;

import org.cs3.jmangler.supplier.UnextendableClassSet;
import org.cs3.jmangler.tau.CodeTransformerComponent;
import org.cs3.jmangler.tau.ExtensionSet;
import org.cs3.jmangler.tau.InterfaceTransformerComponent;
import org.objectteams.transformer.BaseMethodTransformation;
import org.objectteams.transformer.ObjectTeamsTransformation.SharedState;

import de.fub.bytecode.generic.ClassGen;

// ObjectTeams/Java Runtime System.
// Copyright (c) 2002 Christine Hundt, 2003 Stephan Herrmann, 2004 Christin Hundt.
// Project Home Page: http://www.ObjectTeams.org
//
// This program is free software.
//
// You may redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the
// Free Software Foundation. Version 2.1 of the license should be
// included with this distribution in the file LICENSE.LGPL.
// If the license is not included with this distribution,
// you may find a copy at the FSF web site at 'www.gnu.org' or
//  'www.fsf.org', or you may write to the Free Software Foundation,
// 675 Mass Ave, Cambridge, MA 02139 USA.
//
// THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
// NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
// OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
// CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
// REDISTRIBUTION OF THIS SOFTWARE.

/**
 * Insert dispatch code into base methods affected by callin bindings. <p>
 * If a loaded class contains binding declarations (transmitted by attributes)
 * these are stored for further determination of necessity for base-method-transforming.
 * Classes for which a callin binding exists will be transformed:
 * (base-class)methods <i>m()</i> which are changed by a callin will be copied to
 * <tt>_OT$<i>m</i>$orig()</tt>.
 * Hereafter the original method <i>m()</i> will be transformend depending on the
 * callin-modifier:
 * <ul>
 *  <li> replace: only the (role-)callin-method is called
 *  <li> after: first the original-method <tt>_OT$<i>m</i>$orig()</tt> is called and
 *       then the callin-method
 *  <li> before: first the the callin-method is called and then original-method
 *       <tt>_OT$<i>m</i>$orig()</tt>
 * </ul>
 *
 * @version $Id: BaseMethodTransformer.java,v 1.4 2006-10-09 16:00:05 resix Exp $
 * @author  Christine Hundt, Stephan Herrmann
 */

public class BaseMethodTransformer
	extends ObjectTeamsTransformer
	implements InterfaceTransformerComponent, CodeTransformerComponent
{
    
    private BaseMethodTransformation transformation = new BaseMethodTransformation(new SharedState()); 
    
	/**
	 *  The code transformer only replaces the original code with
	 *  the initial wrapper.
	 */
    public void transformCode(UnextendableClassSet cs) {
        Iterator iter = cs.getIteratorForTransformableClasses();
        while (iter.hasNext()) {
        // look at every transformable class
            ClassGen        cg         = (ClassGen)iter.next();
            
            transformation.doTransformCode(cg);
        }
    }

	/**
	 * Main entry for this transformer.
	 */
    public ExtensionSet transformInterface(UnextendableClassSet cs) {
        Iterator iter = cs.getIteratorForTransformableClasses();
        ExtensionSet es = new ExtensionSet();
         JManglerEnhancer jme = new JManglerEnhancer(this, es, cs);

        while (iter.hasNext()) {
            // look at every transformable class
            ClassGen        cg         = (ClassGen)iter.next();
            
            transformation.doTransformInterface(jme, cg);
        }
        // return the changes
        return es;
    }
}	
