package org.objectteams.transformer.jmangler;

import java.util.Iterator;

import org.cs3.jmangler.supplier.UnextendableClassSet;
import org.cs3.jmangler.tau.CodeTransformerComponent;
import org.cs3.jmangler.tau.ExtensionSet;
import org.cs3.jmangler.tau.InterfaceTransformerComponent;
import org.objectteams.transformer.BaseTagInsertion;
import org.objectteams.transformer.BaseTagInsertion.SharedState;

import de.fub.bytecode.generic.ClassGen;

//ObjectTeams/Java Runtime System.
//Copyright (c) 2002 Christine Hundt, 2003 Stephan Herrmann, 2004 Christine Hundt.
//Project Home Page: http://www.ObjectTeams.org
//
//This program is free software.
//
//You may redistribute it and/or modify it under the terms of the
//GNU Lesser General Public License as published by the
//Free Software Foundation. Version 2.1 of the license should be
//included with this distribution in the file LICENSE.LGPL.
//If the license is not included with this distribution,
//you may find a copy at the FSF web site at 'www.gnu.org' or
//'www.fsf.org', or you may write to the Free Software Foundation,
//675 Mass Ave, Cambridge, MA 02139 USA.
//
//THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
//NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
//OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
//CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
//REDISTRIBUTION OF THIS SOFTWARE.

/**
 * This transformer inserts tag fields for all (topmost) bound base classes.
 * Tag fields look like this:
 * <pre>
 *   public short <i>TeamName</i>_OT$Tag;
 * </pre>
 * where <i>TeamName</i> is any team which has a callin binding for this base class.
 * It also inserts initialization of this tag into every constructor.
 * 
 * @version $Id: BaseTagInserter.java,v 1.3 2005-08-02 09:28:37 resix Exp $
 * @author  <Stephan Herrmann
 */
public class BaseTagInserter 
    implements InterfaceTransformerComponent, CodeTransformerComponent 
{

    private BaseTagInsertion transformation = new BaseTagInsertion(new SharedState());
	
    public ExtensionSet transformInterface(UnextendableClassSet cs) {

        Iterator   iter = cs.getIteratorForTransformableClasses();
        ExtensionSet es = new ExtensionSet();
        JManglerEnhancer jme = new JManglerEnhancer(this, es, cs);
        
        while (iter.hasNext()) {
            // look at every transformable class
            ClassGen cg = (ClassGen)iter.next();
            
            transformation.doTransformInterface(jme, cg);
        }
        // return the changes
        return es;
    }



    /**
     *  Insert tag initializations into each constructor.
     */ 
    public void transformCode(UnextendableClassSet cs) {
 
        Iterator iter = cs.getIteratorForTransformableClasses();
        while (iter.hasNext()) {
            // look at every transformable class
            ClassGen cg = (ClassGen)iter.next();
            
            transformation.doTransformCode(cg);
        }
    }

 
    public void sessionStart() {
        //System.out.println(" BaseCallinRedirector Session starts.");
    }

    public void sessionEnd() {
        //System.out.println(" BaseCallinRedirector Session ends.");
    }

    public java.lang.String verboseMessage() {
        // give the class name in startup message
        return toString();
    }    


}
