package org.objectteams.transformer.jmangler;

import java.util.Iterator;

import org.cs3.jmangler.supplier.UnextendableClassSet;
import org.cs3.jmangler.tau.CodeTransformerComponent;
import org.cs3.jmangler.tau.ExtensionSet;
import org.cs3.jmangler.tau.InterfaceTransformerComponent;
import org.objectteams.transformer.StaticSliceBaseTransformation;
import org.objectteams.transformer.ObjectTeamsTransformation.SharedState;

import de.fub.bytecode.generic.ClassGen;

//ObjectTeams/Java Runtime System.
//Copyright (c) 2002 Christine Hundt, 2003 Stephan Herrmann, 2004 Christine Hundt.
//Project Home Page: http://www.ObjectTeams.org
//
//This program is free software.
//
//You may redistribute it and/or modify it under the terms of the
//GNU Lesser General Public License as published by the
//Free Software Foundation. Version 2.1 of the license should be
//included with this distribution in the file LICENSE.LGPL.
//If the license is not included with this distribution,
//you may find a copy at the FSF web site at 'www.gnu.org' or
//'www.fsf.org', or you may write to the Free Software Foundation,
//675 Mass Ave, Cambridge, MA 02139 USA.
//
//THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
//NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
//OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
//CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
//REDISTRIBUTION OF THIS SOFTWARE.

/**
 * Adds to base classes with callin bindings what can be literally pasted in:
 * 
 * Fields:
 * 		protected static Team[] _OT$activeTeams;
 * 		protected static int[] _OT$activeTeamIDs;
 * Methods:
 * 		public static void _OT$addTeam(Team team, in team_id)
 * 		public static void _OT$removeTeam(Team team)
 * Static initialization (adds clinit method, if not yet presented):
 * 		_OT$activeTeams = new Team[0];
 * 		_OT$activeTeamIDs = new int[0];
 * 
 *   
 * @version $Id: StaticSliceBaseTransformer.java,v 1.3 2005-08-02 09:28:37 resix Exp $
 * @author  <A> Christine Hundt</A>
 */


public class StaticSliceBaseTransformer 
	extends ObjectTeamsTransformer 
	implements InterfaceTransformerComponent, CodeTransformerComponent 
{

	private StaticSliceBaseTransformation transformation = new StaticSliceBaseTransformation(new SharedState());
	
	public void transformCode(UnextendableClassSet cs) {
		
        Iterator iter = cs.getIteratorForTransformableClasses();
        while (iter.hasNext()) {
        // look at every transformable class
            ClassGen cg = (ClassGen)iter.next();
            transformation.doTransformCode(cg);
        }
     }


	public ExtensionSet transformInterface(UnextendableClassSet cs) {
		
        Iterator iter = cs.getIteratorForTransformableClasses();
        ExtensionSet es = new ExtensionSet();
        JManglerEnhancer jme = new JManglerEnhancer(this, es, cs);

        while (iter.hasNext()) {
            // look at every transformable class
            ClassGen cg = (ClassGen)iter.next();
            transformation.doTransformInterface(jme, cg);
        }
        // return the changes
        return es;
    }


    
}
