package org.objectteams.transformer.jmangler;

import java.util.Iterator;

import org.cs3.jmangler.supplier.UnextendableClassSet;
import org.cs3.jmangler.tau.CodeTransformerComponent;
import org.cs3.jmangler.tau.ExtensionSet;
import org.cs3.jmangler.tau.InterfaceTransformerComponent;
import org.objectteams.transformer.TeamInterfaceImplementation;
import org.objectteams.transformer.ObjectTeamsTransformation.SharedState;

import de.fub.bytecode.generic.ClassGen;

//ObjectTeams/Java Runtime System.
//Copyright (c) 2002 Christine Hundt, 2003 Stephan Herrmann, 2004 Christine Hundt.
//Project Home Page: http://www.ObjectTeams.org
//
//This program is free software.
//
//You may redistribute it and/or modify it under the terms of the
//GNU Lesser General Public License as published by the
//Free Software Foundation. Version 2.1 of the license should be
//included with this distribution in the file LICENSE.LGPL.
//If the license is not included with this distribution,
//you may find a copy at the FSF web site at 'www.gnu.org' or
//'www.fsf.org', or you may write to the Free Software Foundation,
//675 Mass Ave, Cambridge, MA 02139 USA.
//
//THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
//NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
//OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
//CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
//REDISTRIBUTION OF THIS SOFTWARE.

/**
 * Adds the general Team infrastructure to team classes.
 * 
 * Fields:
 * 		public (static) final_OT$ID <- static if 'ot.no_static' ist set
 *  Methods:
 * 		public int activate(int level)
 * 		public int deactivate(int level)
 * 		public int _OT$getID()
 * Static initialization (adds clinit method, if not yet presented) OR to every constructor: <- if 'ot.no_static' is set
 * 		_OT$ID = TeamIdDispenser.getTeamId(class_name) 
 * 
 * Enables implicit team activation for normal (user defined) public team-level methods.
 * A call to 'activate' is woven at the beginning of the method.
 * A call to 'deactivate' is woven before every return and at every thrown exception.
 *
 * @version $Id: TeamInterfaceImplementer.java,v 1.3 2005-08-02 09:28:37 resix Exp $
 * @author  <A> Christine Hundt</A>
 */


public class TeamInterfaceImplementer 
	extends ObjectTeamsTransformer 
	implements InterfaceTransformerComponent, CodeTransformerComponent 
{

	private TeamInterfaceImplementation transformation = new TeamInterfaceImplementation(false, new SharedState());
    
	public void transformCode(UnextendableClassSet cs) {

        Iterator iter = cs.getIteratorForTransformableClasses();
        while (iter.hasNext()) {
        // look at every transformable class
            ClassGen cg = (ClassGen)iter.next();
            transformation.doTransformCode(cg);
        }
     }



	public ExtensionSet transformInterface(UnextendableClassSet cs) {
		
        Iterator iter = cs.getIteratorForTransformableClasses();
        ExtensionSet es = new ExtensionSet();
        JManglerEnhancer jme = new JManglerEnhancer(this, es, cs);
        
        while (iter.hasNext()) {
            // look at every transformable class
            ClassGen cg = (ClassGen)iter.next();
            transformation.doTransformInterface(jme, cg);
        }
        // return the changes
        return es;
    }


}
