package org.objectteams.transformer.util;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 *
 */
public class AttributeReadingGuard {
	
	private static Map<ClassLoader, AttributeReadingGuard> instances = new HashMap<ClassLoader, AttributeReadingGuard>();
    private static AttributeReadingGuard defaultInstance = new AttributeReadingGuard();
    
    private ArrayList<String> servedClasses = new ArrayList<String>();
    
    // this one flag is really global (concerns the one main class of the application):
    private static boolean firstLoaded = true;

    /**
     * @param className
     * @return
     */
    public boolean iAmTheFirst(String className) {
        return !this.servedClasses.contains(className);    
    }
    
    /**
     * Processing the given class is done.
     * @param className
     */
    public void workDone(String className) {
    	this.servedClasses.add(className);    
    }
    
    /**
     * @return whether this class is the first being loaded => possibly the main class.
     */
    public static synchronized boolean isFirstLoadedClass() {
    	if (!firstLoaded)
    		return false;
    	firstLoaded = false;
    	return true;
    }

    /** First loaded class has no main => it was a false alarm. */
	public static void reset() {
		firstLoaded = true;
	}

	/**
	 * Since actual data are stored in an instance, static methods need to retrieve the appropriate
     * instance regarding the given class loader.
	 */
	public static AttributeReadingGuard getInstanceForLoader(ClassLoader loader) {
		if (loader == null)
			return defaultInstance;
		
		AttributeReadingGuard instance = instances.get(loader);
		if (instance == null)
			instances.put(loader, instance = new AttributeReadingGuard());
		return instance;
	}
}
